#include "Game.h"
#include "Bomber.h"
#include "GameObject.h"
#include "Rectangle.h"
#include "InputHandler.h"
#include "FlyingNum.h"
#include "Bucket.h"
#include "Bomb.h"
#include "BombHandler.h"
#include "Collision.h"
#include "Player.h"
#include "Vector2D.h"
#include "Background.h"
#include "SoundManager.h"
#include <iostream>
#include <cstdlib>

#ifndef DATADIR
#define DATADIR ""
#define SEPARATOR ""
#else
#define SEPARATOR "/"
#endif

Game* Game::s_pInstance = 0;

Game::Game(): m_bombHit(false), m_resetScore(false)
{}

int const Game::bombs_per_level[8] = {
    10,
    20,
    30,
    40,
    50,
    75,
    100,
    150
  };

void Game::update()
{

	switch (state) {
	case PAUSE:
		m_press_fire->setActive(true);
		if (TheInputHandler::Instance()->getMouseButtonState(LEFT)) {
			  SDL_Rect rect1;
			  rect1.x = m_press_fire->getPosition().getX();
			  rect1.y = m_press_fire->getPosition().getY();
//			  cout << "fire rect y " << m_press_fire->getPosition().getY() << endl;
			  rect1.w = m_press_fire->getWidth();
//			  cout << "fire rect height " << m_press_fire->getHeight() << endl;
			  rect1.h = m_press_fire->getHeight();

			  Vector2D* mousePos =  TheInputHandler::Instance()->getUnScaleMousePosition();

			  if (m_buckets.empty() || m_level > 8) {
					m_buckets.push_back(m_bucket1);
					m_buckets.push_back(m_bucket2);
					m_buckets.push_back(m_bucket3);
					m_resetScore = true;
					m_level = 1;
			  }

			  if (vector2DRect(mousePos, &rect1)) {
					m_bomber->setLevel(m_level);
					m_bomber->setNumBombs(bombs_per_level[m_level - 1]);
					//m_bomber->setBombLastTime();
					m_bombsCaught = 0;
					//TheSoundManager::Instance()->playMusic("bizjung", -1);

					state = PLAY;
			  }

		}
		break;
	case PLAY:
		if (m_resetScore) {
			m_totScore = 0;
			m_level = 1;
			m_resetScore = false;
		}
		if (m_bombsCaught == bombs_per_level[m_level - 1]) {
			//TheSoundManager::Instance()->pauseMusic();
			state = PAUSE;
			cout << "before advance level " << m_level << endl;
			m_level++;
			fly_num->activate(m_level);
			cout << "level after " << m_level << endl;
			m_bombsCaught = 0;
		}
		m_press_fire->setActive(false);
		if (m_bombHit) {
			TheBombHandler::Instance()->startExploding();
			m_buckets.pop_back();
			TheSoundManager::Instance()->pauseMusic();
			state = EXPLODE;
			// reset flag
			m_bombHit = false;
		}
		break;
	case EXPLODE:
		m_press_fire->setActive(false);
		if (m_bombsEmpty) {
			state = PAUSE;
			TheSoundManager::Instance()->resumeMusic();
			m_bombsEmpty = false;
		}
		break;

	}
  // loop through and update our objects
  for(std::vector<GameObject*>::size_type i = 0; i !=   m_gameObjects.size(); i++)
  {
    m_gameObjects[i]->update();
  }
  for(std::vector<Bucket*>::size_type i = 0; i !=   m_buckets.size(); i++)
  {
    m_buckets[i]->update();
  }
  //cout << "update bomber" << endl;
  m_bomber->update();
  m_score->setScore(m_totScore);

  // check for collisions
  std::vector<Bomb*> bombs = BombHandler::Instance()->getBombs();
  //m_buckets
  // use iterator like in bombhandler

  for (std::vector<Bomb*>::iterator p_it = bombs.begin(); p_it != bombs.end();) {
	  bool collide = false;
	  SDL_Rect rect1;
	  rect1.x = (*p_it)->getPosition().getX();
	  rect1.y = (*p_it)->getPosition().getY();
	  rect1.w = (*p_it)->getWidth();
	  rect1.h = (*p_it)->getHeight();

	  for(std::vector<Bucket *>::iterator b_it = m_buckets.begin() ; b_it != m_buckets.end() ;)
	  {
		  SDL_Rect rect2;
		  rect2.x = (*b_it)->getPosition().getX();
		  rect2.y = (*b_it)->getPosition().getY();
		  rect2.w = (*b_it)->getWidth();
		  rect2.h = (*b_it)->getHeight();
		  if (RectRect(&rect1,&rect2)) {
			  collide = true;
			  (*p_it)->setCaught();
			  m_totScore++;

		  }
		  ++b_it;

	  }

	  ++p_it;

  }
  if (TheInputHandler::Instance()->isKeyDown(SDL_SCANCODE_ESCAPE)) {
	  m_bRunning = false;
  }


}

bool Game::init(const char* title, int xpos, int ypos, int width, int height, bool fullscreen)

{

	  m_gameWidth = width;
	  m_gameHeight = height;

	  srand(time(NULL));


  int flags = 0;

  if(fullscreen)
  {
    flags = SDL_WINDOW_FULLSCREEN;
  }

  // attempt to initialize SDL
  if(SDL_Init(SDL_INIT_EVERYTHING) == 0)
  {
	  SDL_Rect rect;
	  int rtn = SDL_GetDisplayBounds(0, &rect);
	  if (rtn == 0 && ((flags & SDL_WINDOW_FULLSCREEN) == SDL_WINDOW_FULLSCREEN)) {
	    printf("screen width is %d\n", rect.w);
	    printf("screen height is %d\n", rect.h);
	    m_scaleWidth = rect.w;
	    m_scaleHeight = rect.h;
	  } else {
		    m_scaleWidth = width;
		    m_scaleHeight = height;
	    printf("Did not return a value. %s\n",SDL_GetError());
	  }

    std::cout << "SDL init success\n";
    // init the window
    m_pWindow = SDL_CreateWindow(title, xpos, ypos, 
    		m_scaleWidth, m_scaleHeight, flags);

    if(m_pWindow != 0) // window init success
    {
      std::cout << "window creation success\n";
      m_pRenderer = SDL_CreateRenderer(m_pWindow, -1, 0);

      if(m_pRenderer != 0) // renderer init success
      {
        std::cout << "renderer creation success\n";
        SDL_SetRenderDrawColor(m_pRenderer, 0,0,0,255);

	//SDL_Surface* pTempSurface = SDL_LoadBMP("./assets/rider.bmp");
		//SDL_Surface* pTempSurface = SDL_LoadBMP("./assets/animate.bmp");
		//SDL_Surface* pTempSurface = IMG_Load("./assets/animate-alpha.png");

		//m_textureManager.load("assets/animate-alpha.png", "animate", m_pRenderer);



	if(!TheTextureManager::Instance()->load(DATADIR SEPARATOR "background.png", "background", m_pRenderer))
	{
	   return false;
	}

	if(!TheTextureManager::Instance()->load(DATADIR SEPARATOR "bomber-happy.png", "bomber-happy", m_pRenderer))
	{
	   return false;
	}

	if(!TheTextureManager::Instance()->load(DATADIR SEPARATOR "bucket_sheet.png", "bucket", m_pRenderer))
	{
	   return false;
	}

	if(!TheTextureManager::Instance()->load(DATADIR SEPARATOR "bomb_sheet.png", "bomb", m_pRenderer))
	{
	   return false;
	}

	if(!TheTextureManager::Instance()->load(DATADIR SEPARATOR "press-fire.png", "press-fire", m_pRenderer))
	{
	   return false;
	}

	if(!TheTextureManager::Instance()->load(DATADIR SEPARATOR "number_sheet.png", "number_sheet", m_pRenderer))
	{
	   return false;
	}

	if(!TheSoundManager::Instance()->load(DATADIR SEPARATOR "music/bizjung.it", "bizjung", SOUND_MUSIC))
	{
	   cout << "Failed to load bizjung" << endl;
	   return false;
	}

	if(!TheSoundManager::Instance()->load(DATADIR SEPARATOR "sounds/explosion.wav", "explosion", SOUND_SFX))
	{
	   cout << "Failed to explosion effect" << endl;
	   return false;
	}

	state = PAUSE;
	m_level = 1;
	m_bombsCaught = 0;
	m_totScore = 0;

	//m_background = new Background();
	m_background = new Background();
	m_bomber = new Bomber();
	m_rect = new Rectangle();

	m_bucket1 = new Bucket(0);
	m_bucket2 = new Bucket(1);
	m_bucket3 = new Bucket(2);

	// start with digit at level 1
	fly_num = new FlyingNum(m_level);


	m_press_fire = new GameObject();
	m_score = new Score();
	m_score->setScore(0);

	m_score->load("number_sheet");
	m_press_fire->load(258,228,128,24,"press-fire");



	m_bucket1->load(100, 400, 64, 32, "bucket");
	m_bucket2->load(100, 360, 64, 32, "bucket");
	m_bucket3->load(100, 320, 64, 32, "bucket");

	m_background->load(0, 0, 640, 480, "background");

	m_bomber->load(80, 65, 32, 64, "bomber-happy");


	m_gameObjects.push_back(m_background);



	m_buckets.push_back(m_bucket1);
	m_buckets.push_back(m_bucket2);
	m_buckets.push_back(m_bucket3);

	m_gameObjects.push_back(m_press_fire);
	m_gameObjects.push_back(fly_num);

	m_gameObjects.push_back(m_score);

  	TheSoundManager::Instance()->playMusic("bizjung", -1);


      }
      else
      {
        std::cout << "renderer init fail\n";
        return false; // renderer init fail
      }
    }
    else
    {
      std::cout << "window init fail\n";
      return false; // window init fail
    }
  }
  else
  {
    std::cout << "SDL init fail\n";
    return false; // SDL init fail
  }

  std::cout << "init success\n";
  m_bRunning = true; // everything inited successfully, start the main loop

 



  return true;
}

void Game::render()
{
  SDL_RenderClear(m_pRenderer); // clear to the draw colour

  // loop through our objects and draw them
  for(std::vector<GameObject*>::size_type i = 0; i != 
  m_gameObjects.size(); i++)
  {
    m_gameObjects[i]->draw(m_pRenderer);
  }

  // render buckets
  for(std::vector<GameObject*>::size_type i = 0; i !=
  m_buckets.size(); i++)
  {
    m_buckets[i]->draw(m_pRenderer);
  }

  m_bomber->draw(m_pRenderer);

  SDL_RenderPresent(m_pRenderer); // draw to the screen
}

void Game::clean()
{
  std::cout << "cleaning game\n";
  SDL_DestroyWindow(m_pWindow);
  SDL_DestroyRenderer(m_pRenderer);
  SDL_Quit();
}

void Game::handleEvents()
{
	TheInputHandler::Instance()->update();
	if (TheInputHandler::Instance()->quit())
		m_bRunning = false;

}

void Game::draw()
{
  for(std::vector<GameObject*>::size_type i = 0; i !=   m_gameObjects.size(); i++) 
  {
    m_gameObjects[i]->draw(m_pRenderer);
  }
}
